#include <p24HJ64GP202.h>
#define UART_H_MAIN_FILE
#include "auxiliary.h"
#include "uart.h"
#include "it700Driver.h"






/*----------------------------------------------------------------------
* Function name:		UART_Init
* Input:				void
* Output:				void
* Purpose:				Init the UART module.
*-----------------------------------------------------------------------*/
void UART_Init(void)
{
	/* Initialize receive and transmit buffer counters */
	g_strUart.UART_RxTail = 0;
	g_strUart.UART_RxHead = 0;
	g_strUart.UART_TxTail = 0;
	g_strUart.UART_TxHead = 0;
	g_strUart.ERROR = 0;
}

/*----------------------------------------------------------------------
* Function name:		byUART_UART_DataInRxBuf
* Input:				void
* Output:				TRUE/FALSE
* Purpose:				This function checks if there incoming data 
*						pending
*-----------------------------------------------------------------------*/
BYTE byUART_DataInRxBuf( void )
{
 return ( g_strUart.UART_RxHead != g_strUart.UART_RxTail ); // Return FALSE if the receive 	
															// buffer is empty
}

/*----------------------------------------------------------------------
* Function name:		byUART_UART_DataInTxBuf
* Input:				void
* Output:				TRUE/FALSE
* Purpose:				This function checks if there outgoing data 
*						pending
*-----------------------------------------------------------------------*/
BYTE byUART_DataInTxBuf( void )
{	
	return ( g_strUart.UART_TxHead != g_strUart.UART_TxTail );	// Return FALSE if the transmitted
																// buffer is empty             
}
BYTE byUART_IR_DataInTxBuf( void )
{	
	return ( g_strUartIR.UART_IR_TxHead != g_strUartIR.UART_IR_TxTail );	// Return FALSE if the transmitted
																// buffer is empty             
}

/*----------------------------------------------------------------------
* Function name:		byUART_Receive
* Input:				void
* Output:				incoming byte
* Purpose:				Return one incoming byte
*-----------------------------------------------------------------------*/
BYTE byUART_Receive( void )
{
	g_strUart.UART_RxTail = ( g_strUart.UART_RxTail + 1 ) & UART_RX_BUFFER_MASK;  // Store new index
	
	return g_strUart.UART_RxBuf[g_strUart.UART_RxTail]; // Return data 
}

/*----------------------------------------------------------------------
* Function name:		UART_CB_OnRxLocalByte
* Input:				incoming byte
* Output:				void
* Purpose:				Local CB Rx function - stores the incoming
*						byte in local buffer.
*-----------------------------------------------------------------------*/
void UART_CB_OnRxLocalByte(BYTE byData)
{
    BYTE bTmpHead;
	
	bTmpHead = ( g_strUart.UART_RxHead + 1 ) & UART_RX_BUFFER_MASK; // increase buffer index 
	g_strUart.UART_RxHead = bTmpHead; // Store new index    

	if ( bTmpHead == g_strUart.UART_RxTail )
	{
		g_strUart.ERROR = UART_RxBuffer_OverFlow; // ERROR! Receive buffer overflow 
	}
	else // in case buffer is not full
		g_strUart.UART_RxBuf[bTmpHead] = byData; // Store received data in buffer
}

void UART_IR_RxLocalByte(BYTE byData)
{
    WORD byTmpHead;
	
	byTmpHead = ( g_strUartIR.UART_IR_RxHead + 1 ) & UART_IR_RX_BUFFER_MASK; // increase buffer index 
	g_strUartIR.UART_IR_RxHead = byTmpHead; // Store new index    

//	if ( byTmpHead == g_strUartIR.UART_IR_RxTail )
//		g_strUartIR.ERROR_IR = UART_IR_RxBuffer_OverFlow; // ERROR! Receive buffer overflow 
//	else // in case buffer is not full
		g_strUartIR.UART_IR_RxBuf[byTmpHead] = byData;	 // Store received data in buffer de resposta
}



/*----------------------------------------------------------------------
* Function name:		UART_Transmit
* Input:				pointer to data, length
* Output:				void
* Purpose:				this function transmits data via UART. 
*-----------------------------------------------------------------------*/
void UART_Transmit( PBYTE pbyData , WORD wLength )
{
	BYTE texBufArray[wLength];
	BYTE byTmpHead;
	int k;
	
	for(k=0;k<wLength;k++)				//guarda a informaao num array temporario
		texBufArray[k] = pbyData[k];
	
	for(k = 0; k < wLength; k++)
	{
		byTmpHead = ( g_strUart.UART_TxHead + 1 ) & UART_TX_BUFFER_MASK; // increase buffer index 
	
		g_strUart.UART_TxHead = byTmpHead; // Store new index    
	
		if ( byTmpHead == g_strUart.UART_TxTail )
		{
			g_strUart.ERROR = UART_TxBuffer_OverFlow; // ERROR! Transmit buffer overflow 
		}
		else // in case buffer is not full
			g_strUart.UART_TxBuf[byTmpHead] = pbyData[k]; // Store data to Transmit, in buffer
	}

	Nop();
	IFS0bits.U1TXIF = 1;	//dispara propositadamente a interrupcao de USART1-TX. Obriga a envio imediato.
}

void UART_Transmit_IR( PBYTE pbyData , WORD wLength )
{

	WORD wTmpHead;
	WORD k;

	for(k = 0; k < wLength; k++)
	{
		wTmpHead = ( g_strUartIR.UART_IR_TxHead + 1 ) & UART_IR_TX_BUFFER_MASK; // increase buffer index 
	
		g_strUartIR.UART_IR_TxHead = wTmpHead; // Store new index    
	
		if ( wTmpHead == g_strUartIR.UART_IR_TxTail )
			g_strUartIR.ERROR_IR = UART_IR_TxBuffer_OverFlow; // ERROR! Transmit buffer overflow 
		else // in case buffer is not full
			g_strUartIR.UART_IR_TxBuf[wTmpHead] = pbyData[k]; // Store data to Transmit, in buffer
	}

	Nop();
	IFS1bits.U2TXIF = 1;	//dispara propositadamente a interrupcao de USART2-TX. Obriga a envio imediato.
}



