
#ifndef UART_H
#define UART_H

#ifdef  UART_H_MAIN_FILE
#define	UART_H_EXT
#else
#define	UART_H_EXT	extern
#endif

// Defines:
/* UART Buffer Defines */

#define UART_RX_BUFFER_SIZE 256  /* 2,4,8,16,32,64,128 or 256 bytes */
#define UART_TX_BUFFER_SIZE 256   /* 2,4,8,16,32,64,128 or 256 bytes */
#define UART_IR_RX_BUFFER_SIZE 512  /* 2,4,8,16,32,64,128 or 256 bytes */
#define UART_IR_TX_BUFFER_SIZE 128   /* 2,4,8,16,32,64,128 or 256 bytes */

#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )
#define UART_IR_RX_BUFFER_MASK ( UART_IR_RX_BUFFER_SIZE - 1 )
#define UART_IR_TX_BUFFER_MASK ( UART_IR_TX_BUFFER_SIZE - 1 )


#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
	#error RX1 buffer size is not a power of 2
#endif

#if ( UART_TX_BUFFER_SIZE & UART_TX_BUFFER_MASK )
	#error TX1 buffer size is not a power of 2
#endif

// Define Frame Format:
#define NumOf_StopBit    0  

//  mark 0 for 1 stop bit
//  mark 1 for 2 stop bits

#define NumOf_DataBits   3    //number of data bits in a frame.

//	mark 0 for 5 bits
//	mark 1 for 6 bits				 	
//  mark 2 for 7 bits
//  mark 3 for 8 bits
//  mark 7 for 9 bits

// define HAL erors 

#define UART_RxBuffer_OverFlow  0x01  
#define UART_TxBuffer_OverFlow  0x02 			
#define UART_IR_RxBuffer_OverFlow  0x01  
#define UART_IR_TxBuffer_OverFlow  0x02 			


/* Prototypes */

//UART functions:
UART_H_EXT void UART_Init(void);
UART_H_EXT void UART_Transmit(PBYTE data,WORD length);
UART_H_EXT void UART_Transmit_IR(PBYTE pbyData,WORD wLength);
UART_H_EXT BYTE byUART_Receive(void);
UART_H_EXT BYTE byUART_DataInRxBuf(void);
UART_H_EXT BYTE byUART_DataInTxBuf(void);
UART_H_EXT BYTE byUART_IR_DataInTxBuf(void);
UART_H_EXT void UART_CB_OnRxLocalByte(BYTE byData);
UART_H_EXT void UART_IR_RxLocalByte(BYTE byData);



 
struct _UART_GLOBAL_T
  {
	/* UART Variables */
	BYTE  UART_RxBuf[UART_RX_BUFFER_SIZE];	// buffer for storing the incoming bytes from UART interrupt
	BYTE  UART_TxBuf[UART_TX_BUFFER_SIZE];	// buffer for storing the bytes that are waiting to be transfered by UART.
	volatile BYTE  UART_RxHead;		// current index of the last received byte in UART_RxBuf
	volatile BYTE  UART_RxTail; 	// current index of the last free space in the UART_RxBuf.		   		
	volatile BYTE  UART_TxHead; 	// current index of the last byte in UART_TxBuf to transmit.
	volatile BYTE  UART_TxTail; 	// current index of the last transmitted byte in the UART_TxBuf.
		
	volatile BYTE  ERROR;

};
extern struct _UART_GLOBAL_T g_strUart;




struct _UART_IR_GLOBAL_T
{
   	
// UART GPRS Variables 
	BYTE  UART_IR_RxBuf[UART_IR_RX_BUFFER_SIZE];		// buffer for storing the incoming bytes from UART interrupt
//	BYTE  UART_IR_DATA_RxBuf[UART_IR_RX_BUFFER_SIZE];	// buffer for storing the incoming bytes from UART interrupt
	BYTE  UART_IR_TxBuf[UART_IR_TX_BUFFER_SIZE];		// buffer for storing the bytes that are waiting to be transfered by UART.
	BYTE  UART_IR_TxBuf_temp[UART_IR_TX_BUFFER_SIZE];	// buffer for storing the bytes that are waiting to be transfered by UART.
	volatile WORD  UART_IR_RxHead; 	// current index of the last received byte in UART_RxBuf
	volatile WORD  UART_IR_RxTail;	// current index of the last free space in the UART_RxBuf.		   		
	volatile WORD  UART_IR_TxHead;	// current index of the last byte in UART_TxBuf to transmit.
	volatile WORD  UART_IR_TxTail;	// current index of the last transmitted byte in the UART_TxBuf.
	
	volatile BYTE  ERROR_IR;

};
extern struct _UART_IR_GLOBAL_T g_strUartIR;



#endif
